""" Utility functions and classes for SRP

Context : SRP
Module  : Frames.py
Version : 1.9.4
Author  : Stefano Covino
Date    : 31/10/2018
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (27/09/2010) First version.
        : (28/09/2010) Minor name changes.
        : (29/09/2010) Minor correction.
        : (01/10/2010) Warning in saving FITS file suppressed.
        : (03/10/2010) Number of stars for astrometry. RA and DEC shift computed.
        : (04/10/2010) Sources close to frame border cleaned.
        : (13/10/2010) Larger search radius for catalogue object and no more maximum 
        :               number of objects.
        : (14/10/2010) Better estimate of frame size.
        : (24/10/2010) USNO-A2 catalogue also be used.
        : (25/10/2010) Flexible threshold for bad quality frames.
        : (27/10/2010) Simpler comment output.
        : (03/11/2010) Better comment formatting.
        : (17/11/2010) WCS header comments.
        : (20/11/2010) Better header formatting.
        : (31/12/2010) Minor bug correction.
        : (26/08/2011) Consistency check for unreliable CRVAL1,2 and CDELT1,2.
        : (04/11/2011) More flexibility in header coordinate format.
        : (22/01/2012) Check that at least one catalogue returns data.
        : (19/08/2013) Data saved with the original format.
        : (29/08/2013) Tunable tolerance for triangle match. Do not update RA,DEC if they already exist.
        : (03/09/2013) Better search of catalogue objects.
        : (16/10/2013) Better computation of frame center.
        : (16/07/2014) Deal with non standard FITS headers.
        : (30/10/2014) Sextractor source finding.
        : (09/12/2014) Better choice of threshold for sextractor source finding.
        : (12/01/2015) No more del in dictionaries.
        : (31/07/2015) python3 porting.
        : (14/03/2017) Less iterations in astrometry if rotation and/or pizel size is provided.
        : (16/05/2017) Minor update.
        : (18/05/2017) Minor update.
        : (31/10/2018) 'astlib' in computation.
"""


import copy, math, warnings

import astropy.io.fits as aif
import astLib.astCoords as aLaC
import numpy
import scipy.optimize as so

import SRP
from SRP.SRPMath.AstroCoordInput import AstroCoordInput
from SRP.SRPCatalogue.TWOMASSClass import TWOMASS
from SRP.SRPCatalogue.USNOClass import USNO
from SRPFITS.Fits.FitsImageClass import FitsImage
from SRPFITS.Fits import FitsConstants
from SRP.SRPMath.AngularDistance import AngularDistance
from SRP.SRPMath.AstroAngleInput import AstroAngleInput
from SRP.SRPMath.AstroCoordInput import AstroCoordInput
from SRP.SRPMath.TriangleClass import Triangle
from SRPFITS.Math.TriangleMatch import TriangleMatch
from SRP.SRPStatistics.CoordDistanceMinimization import CoordDistanceMinimization
from SRP.SRPStatistics.SkyDistanceMinimization import SkyDistanceMinimization
from SRP.SRPStatistics.SkyDistanceMinimization import SkyDistSum
from SRPFITS.Frames.WCS2Pixel import WCS2Pixel
from SRPFITS.Frames.Pixel2WCS import Pixel2WCS


class Astrometry:
    def __init__ (self, fitsfile, center=None, pixcenter=None, point=None, pixsize=None, rotangle=None, framesize=None, maxres=3.0):
            # Fits file
            self.FitsFrame = FitsImage(fitsfile)
            # WCS
            # Pointing direction
            if point != None:
                pcoord = AstroCoordInput(point[0],point[1])
                self.RA = pcoord.RA
                self.DEC = pcoord.DEC
            else:
                try:
                    heacoord = AstroCoordInput(self.FitsFrame.Header[FitsConstants.RA],self.FitsFrame.Header[FitsConstants.DEC])
                except KeyError:
                    heacoord = AstroCoordInput(0.,0.)
                self.RA = heacoord.RA
                self.DEC = heacoord.DEC
            # Pixel reference
            if pixcenter != None:
                self.CRPIX1 = pixcenter[0]
                self.CRPIX2 = pixcenter[1]
            else:
                try:
                    self.CRPIX1 = self.FitsFrame.Header[FitsConstants.CRPIX1]
                    self.CRPIX2 = self.FitsFrame.Header[FitsConstants.CRPIX2]
                except KeyError:
                    self.CRPIX1 = 0.
                    self.CRPIX2 = 0.
            # Pixel size
            if pixsize != None:
                self.CDELT1 = pixsize[0]
                self.CDELT2 = pixsize[1]
                self.pixsizeprov = True
            else:
                self.pixsizeprov = False
                try:
                    self.CDELT1 = self.FitsFrame.Header[FitsConstants.CDELT1]
                    self.CDELT2 = self.FitsFrame.Header[FitsConstants.CDELT2]                    
                except KeyError:
                    self.CDELT1 = -1./3600.
                    self.CDELT2 = 1./3600.
            # check consistency of pixel size, it should not be larger than 0.1deg
            if math.fabs(self.CDELT1) > 0.1:
                self.CDELT1 = -1./3600.
            if math.fabs(self.CDELT2) > 0.1:
                self.CDELT2 = 1./3600.
            # Rotation angle
            if rotangle != None:
                angrad = math.radians(rotangle)
                self.RotationAngle = rotangle
                self.PC11 = math.cos(angrad)
                self.PC12 = -math.sin(angrad)
                self.PC21 = math.sin(angrad)
                self.PC22 = math.cos(angrad)
                self.pc = True
                self.rotangleprov = True
            else:
                self.rotangleprov = False
                try:
                    self.RotationAngle = self.FitsFrame.WCS.getRotationDeg()
                except AttributeError:
                    self.RotationAngle = 0.0
                try:
                    self.PC11 = self.FitsFrame.Header[FitsConstants.PC11]
                    self.PC12 = self.FitsFrame.Header[FitsConstants.PC12]
                    self.PC21 = self.FitsFrame.Header[FitsConstants.PC21]
                    self.PC22 = self.FitsFrame.Header[FitsConstants.PC22]
                    self.pc = True
                except KeyError:
                    self.PC11 = 1.
                    self.PC12 = 0.
                    self.PC21 = 0.
                    self.PC22 = 1.
                    self.pc = False
                try:
                    self.CD11 = self.FitsFrame.Header[FitsConstants.CD11]
                    self.CD12 = self.FitsFrame.Header[FitsConstants.CD12]
                    self.CD21 = self.FitsFrame.Header[FitsConstants.CD21]
                    self.CD22 = self.FitsFrame.Header[FitsConstants.CD22]
                    self.cd = True
                except KeyError:
                    self.cd = False
            if self.pc == False and self.cd == True:
                self.PC11 = self.CD11/self.CDELT1
                self.PC12 = self.CD12/self.CDELT2
                self.PC21 = self.CD21/self.CDELT1
                self.PC22 = self.CD22/self.CDELT2
            # Coordinate reference
            if center != None:
                coord = AstroCoordInput(center[0],center[1])
                self.CRVAL1 = coord.RA
                self.CRVAL2 = coord.DEC
            else:
                try:
                    heacoord = AstroCoordInput(self.FitsFrame.Header[FitsConstants.CRVAL1],self.FitsFrame.Header[FitsConstants.CRVAL2])
                    self.CRVAL1 = heacoord.RA
                    self.CRVAL2 = heacoord.DEC
                except KeyError:
                    self.CRVAL1 = self.RA
                    self.CRVAL2 = self.DEC
            # Check consistency between RA,DEC and CRVAL1,2
            if AngularDistance((self.RA,self.DEC),(self.CRVAL1,self.CRVAL2)) > 1.5:
                self.CRVAL1 = self.RA
                self.CRVAL2 = self.DEC
            # Image size
            sizelistpix = self.FitsFrame.GetFrameSizePix()
            if framesize != None:
                self.HalfSize = framesize/2.
            else:
                pxs = sum([math.fabs(i) for i in (self.CDELT1,self.CDELT2)])/2.0
                self.HalfSize = sum([i*pxs/2.0 for i in sizelistpix])/2.0
            #
            # Axes type
            try:
                self.CTYPE1 = self.FitsFrame.Header[FitsConstants.CTYPE1]
                self.CTYPE2 = self.FitsFrame.Header[FitsConstants.CTYPE2]
            except KeyError:
                self.CTYPE1 = FitsConstants.RAType
                self.CTYPE2 = FitsConstants.DECType
            # Strange CTYPES
            if self.CTYPE1 != FitsConstants.RAType or self.CTYPE1 != FitsConstants.DECType and self.CTYPE2 != FitsConstants.RAType or self.CTYPE2 != FitsConstants.DECType:
                self.CTYPE1 = FitsConstants.RAType
                self.CTYPE2 = FitsConstants.DECType   
            # Frame center
            self.FrameCenter = [(i/2.)+1.0 for i in sizelistpix]
            #
            self.List = []
            self.CatList = []
            self.Astrometrized = False
            self.Residuals = -1
            #
            self.MaxRes = maxres
            self.FitAstro = False
            self.TriAstro = False
            # Algorithms
            self.NativeSources = False
            self.EclipseSources = False
            self.SexSources = False
            #
            self.DefineWCSHeader()
  
  
    def GetFrameCenterCoords(self):
            cc = self.FitsFrame.GetFrameCenterPix()
            cpos = Pixel2WCS(self.FitsFrame.Header,[cc])
            return cpos[0]
  
  
    def GetSources(self, maxobjs=15, thre=23, mincnnt=3, cleanperc=1.):
            soglia = copy.copy(thre)
            while len(self.List) < maxobjs and soglia > 1: 
                self.FitsFrame.Sources(soglia,mincnnt)
                self.FitsFrame.CleanBorderSources(cleanperc)
                self.FitsFrame.SortSourceList()
                if len(self.FitsFrame.List) >= maxobjs:
                    self.List = self.FitsFrame.List[:maxobjs]
                else:
                    if soglia-5 > 3:
                        soglia = soglia - 5
                    elif soglia > 3:
                        soglia = 3
                    elif 2 < soglia <= 3:
                        soglia = soglia - 0.5
                    else:
                        soglia = soglia - 0.25
                    self.List = self.FitsFrame.List
            self.NativeSources = True
            self.EclipseSources = False
            self.SexSources = False


    def GetEclipseSources(self, maxobjs=15, thre=22, cleanperc=1.):
            soglia = copy.copy(thre)
            while len(self.List) < maxobjs and soglia > 0.1:
                self.FitsFrame.EclipseSources(soglia)
                self.FitsFrame.CleanBorderSources(cleanperc)
                self.FitsFrame.SortSourceList()
                if len(self.FitsFrame.List) >= maxobjs:
                    self.List = self.FitsFrame.List[:maxobjs]
                else:
                    if soglia-5 > 2:
                        soglia = soglia - 5
                    elif soglia > 2:
                        soglia = 2
                    elif 0.5 < soglia <= 2:
                        soglia = soglia - 0.5
                    else:
                        soglia = soglia - 0.1
                    self.List = self.FitsFrame.List
            self.EclipseSources = True
            self.NativeSources = False
            self.SexSources = False



    def GetSexSources(self, maxobjs=15, thre=100, cleanperc=1.):
            soglia = copy.copy(thre)
            while len(self.List) < maxobjs and soglia > 0.1:
                self.FitsFrame.SexSources(soglia)
                self.FitsFrame.CleanBorderSources(cleanperc)
                self.FitsFrame.SortSourceList()
                if len(self.FitsFrame.List) >= maxobjs:
                    self.List = self.FitsFrame.List[:maxobjs]
                else:
                    if soglia > 10:
                        soglia = soglia - 5
                    elif 3 <= soglia < 10:
                        soglia = soglia - 1
                    elif 1 <= soglia < 3:
                        soglia = soglia - 0.5
                    else:
                        soglia = soglia - 0.1
                    self.List = self.FitsFrame.List
            self.SexSources = True
            self.NativeSources = False
            self.EclipseSources = False


    def GetCatSources(self, maxentr=15, catq='N'): 
            radius = 1.5*self.HalfSize*60.0  # arcmin
            #cntfld = (self.RA,self.DEC)
            crd = self.GetFrameCenterCoords()
            inpcoord = AstroCoordInput(crd[0],crd[1])
            cntfld = (inpcoord.RA, inpcoord.DEC)
            #print cntfld
            #
            if catq == 'N':
                cat = TWOMASS(cntfld[0], cntfld[1], math.floor(radius))
            elif catq == 'O':
                cat = USNO(cntfld[0], cntfld[1], math.floor(radius))
            else:
                cat = USNO(cntfld[0], cntfld[1], math.floor(radius))
            #
            if cat != None:
                cat.GetData()
                cat.sort()
                if len(cat.ListEntries) >= maxentr:
                    self.CatList = cat.ListEntries[:maxentr]
                else:
                    self.CatList = cat.ListEntries    


    def DefineWCSHeader(self):
        # remove alternate WCS
        self.FitsFrame.Header.pop(FitsConstants.CD11,None)
        self.FitsFrame.Header.pop(FitsConstants.CD12,None)
        self.FitsFrame.Header.pop(FitsConstants.CD21,None)
        self.FitsFrame.Header.pop(FitsConstants.CD22,None)
        # center ref
        self.FitsFrame.Header.set(FitsConstants.CRVAL1,self.CRVAL1,FitsConstants.CRVALCmt)
        self.FitsFrame.Header.set(FitsConstants.CRVAL2,self.CRVAL2,FitsConstants.CRVALCmt)
        #             
        self.FitsFrame.Header.set(FitsConstants.CTYPE1,self.CTYPE1,FitsConstants.CTYPECmt)
        self.FitsFrame.Header.set(FitsConstants.CTYPE2,self.CTYPE2,FitsConstants.CTYPECmt)
        # rotation
        self.FitsFrame.Header.set(FitsConstants.PC11,self.PC11,FitsConstants.PCCmt)
        self.FitsFrame.Header.set(FitsConstants.PC12,self.PC12,FitsConstants.PCCmt)
        self.FitsFrame.Header.set(FitsConstants.PC21,self.PC21,FitsConstants.PCCmt)
        self.FitsFrame.Header.set(FitsConstants.PC22,self.PC22,FitsConstants.PCCmt)
        # pixel size
        self.FitsFrame.Header.set(FitsConstants.CDELT1,self.CDELT1,FitsConstants.CDELT1Cmt)
        self.FitsFrame.Header.set(FitsConstants.CDELT2,self.CDELT2,FitsConstants.CDELT2Cmt)
        # pixel ref
        self.FitsFrame.Header.set(FitsConstants.CRPIX1,self.CRPIX1,FitsConstants.CRPIX1Cmt)
        self.FitsFrame.Header.set(FitsConstants.CRPIX2,self.CRPIX2,FitsConstants.CRPIX1Cmt)
        # pointing dir
        if not (FitsConstants.RA in list(self.FitsFrame.Header.keys())):
            self.FitsFrame.Header.set(FitsConstants.RA,self.RA)
        if not (FitsConstants.DEC in list(self.FitsFrame.Header.keys())):
            self.FitsFrame.Header.set(FitsConstants.DEC,self.DEC)
        self.FitsFrame.Header.set(FitsConstants.RADECSys,FitsConstants.RADECSysVal)
        self.FitsFrame.Header.set(FitsConstants.AngUnit1,FitsConstants.AngUnit1Val,FitsConstants.AngUnCmt)
        self.FitsFrame.Header.set(FitsConstants.AngUnit2,FitsConstants.AngUnit2Val,FitsConstants.AngUnCmt)
        #
        self.FitsFrame.WCS.updateFromHeader()


    def UpgradeWCSHeader(self, xxx_todo_changeme):
        (crpix1,crpix2,rot,sf) = xxx_todo_changeme
        rotrnd = math.radians(rot)
        self.FitsFrame.Header.set(FitsConstants.PC11,math.cos(rotrnd))
        self.FitsFrame.Header.set(FitsConstants.PC12,-math.sin(rotrnd))
        self.FitsFrame.Header.set(FitsConstants.PC21,math.sin(rotrnd))
        self.FitsFrame.Header.set(FitsConstants.PC22,math.cos(rotrnd))
        if self.CDELT1 >= 0:
            cdelt1 = sf
        else:
            cdelt1 = -sf
        if self.CDELT2 >= 0:
            cdelt2 = sf
        else:
            cdelt2 = -sf
        self.FitsFrame.Header.set(FitsConstants.CDELT1,cdelt1)
        self.FitsFrame.Header.set(FitsConstants.CDELT2,cdelt2)
        self.FitsFrame.Header.set(FitsConstants.CRPIX1,crpix1)
        self.FitsFrame.Header.set(FitsConstants.CRPIX2,crpix2)
        self.FitsFrame.WCS.updateFromHeader()            
  

    
    def ImproveWCS (self, xxx_todo_changeme1):
        (sx,sy,rot,sf) = xxx_todo_changeme1
        self.CDELT1 = self.CDELT1 * sf
        self.CDELT2 = self.CDELT2 * sf
        self.CRPIX1 = self.CRPIX1 + sx
        self.CRPIX2 = self.CRPIX2 + sy
        self.RotationAngle = self.RotationAngle + rot
                    
                                    
    def FindSolution(self,angtol=0.5,disttol=5.):
        reflist = []
        objlist = []
        # centering
        if self.NativeSources:
            for i in self.List:
                objlist.append(((i[0]-self.CRPIX1),(i[1]-self.CRPIX2)))
                #print objlist[-1]
        elif self.EclipseSources:
            for i in self.List:
                objlist.append(((i.X-self.CRPIX1),(i.Y-self.CRPIX2)))
                #print objlist[-1]
                #print i.flux
        elif self.SexSources:
            for i in self.List:
                objlist.append(((i.X-self.CRPIX1),(i.Y-self.CRPIX2)))
                #print objlist[-1]
                #print i.flux
        #print 
        radec = []
        for i in self.CatList:
            radec.append((i.RA,i.DEC))
            #print i.H
        RefPos = WCS2Pixel(self.FitsFrame.Header,[(self.CRVAL1,self.CRVAL2)],'astlib')
        #print RefPos
        CatListPix = WCS2Pixel(self.FitsFrame.Header,radec,'astlib')
        for i in CatListPix:
            reflist.append(((i[0]-RefPos[0][0]), (i[1]-RefPos[0][1])))
            #print reflist[-1]
        #
        self.REFCDR = reflist
        self.OBJCDR = objlist
        if self.pixsizeprov and self.rotangleprov:
            mtchres = TriangleMatch(reflist,objlist,angtol,disttol,TrueSize=abs(self.CDELT1)*3600.,TrueAng=self.RotationAngle)
        elif self.pixsizeprov and not self.rotangleprov:
            mtchres = TriangleMatch(reflist,objlist,angtol,disttol,TrueSize=abs(self.CDELT1)*3600.)
        elif not self.pixsizeprov and self.rotangleprov:
            mtchres = TriangleMatch(reflist,objlist,angtol,disttol,TrueAng=self.RotationAngle)
        else:
            mtchres = TriangleMatch(reflist,objlist,angtol,disttol)
        if mtchres != None:
#            print mtchres
            mtchinplist = []
            mtchoutlist = []
            for i in mtchres[0]:
                if self.NativeSources:
                    mtchinplist.append((self.List[i][0],self.List[i][1]))
                elif self.EclipseSources:
                    mtchinplist.append((self.List[i].X,self.List[i].Y))
                elif self.SexSources:
                    mtchinplist.append((self.List[i].X,self.List[i].Y))
            for i in mtchres[1]:
                mtchoutlist.append((self.CatList[i].RA,self.CatList[i].DEC))
            self.NStarRes = len(mtchres[0])
            self.ImproveWCS(mtchres[2][0])
            ris = SkyDistSum((self.CRPIX1,self.CRPIX2,self.RotationAngle,math.fabs(self.CDELT1)),mtchinplist,mtchoutlist,self)/len(mtchoutlist)
            dataris = (self.CRPIX1,self.CRPIX2,self.RotationAngle,math.fabs(self.CDELT1))
#            print self.CRVAL1,self.CRVAL2,self.CRPIX1,self.CRPIX2,self.PC11,self.PC12,self.PC21,self.PC22,self.CDELT1,self.CDELT2
            res = SkyDistanceMinimization((self.CRPIX1,self.CRPIX2,self.RotationAngle,math.fabs(self.CDELT1)),mtchinplist,mtchoutlist,self)           
            self.Residuals = res[1]
            self.UpgradeWCSHeader(res[0])
#            print self.CRVAL1,self.CRVAL2,self.CRPIX1,self.CRPIX2,self.PC11,self.PC12,self.PC21,self.PC22,self.CDELT1,self.CDELT2
#            print ris,res[1]
            if self.Residuals <= self.MaxRes:
                self.Astrometrized = True
                self.FitAstro = True
            else:
                # try with no fit
                if ris < self.MaxRes:
                    self.Residuals = ris
                    self.UpgradeWCSHeader(dataris)
                    self.Astrometrized = True
                    self.TriAstro = True 
                else:
                    self.Astrometrized = False
            #
            self.GetPointingShift()
            #
        else:
            self.Astrometrized = False
            self.Residuals = -1
            self.NStarRes = 0
            


    def GetPointingShift(self):
        # derive true center coordinate
        ccord = Pixel2WCS(self.FitsFrame.Header,[(self.FrameCenter[0],self.FrameCenter[1])])
#        self.RAshift = ((self.RA - ccord[0][0])*math.cos(math.radians(self.DEC)))*3600.0
#        self.DECshift = (self.DEC - ccord[0][1])*3600.0
        self.RAshift = ((self.FitsFrame.Header[FitsConstants.RA] - ccord[0][0])*math.cos(math.radians(self.FitsFrame.Header[FitsConstants.DEC])))*3600.0
        self.DECshift = (self.FitsFrame.Header[FitsConstants.DEC] - ccord[0][1])*3600.0

            
    def SaveFile(self,nfile):
        frm = aif.open(self.FitsFrame.Name,do_not_scale_image_data=True)
        if self.Astrometrized:
            self.FitsFrame.Header.add_comment('SRPComment: Astrometric solution acceptable.') 
            self.FitsFrame.Header.add_comment('SRPComment: Average residual: %.2g arcsec for %d stars.' % (self.Residuals, self.NStarRes))  
            self.FitsFrame.Header.add_comment('SRPComment: Shift wrt pointing RA and DEC coords: %.1f %.1f arcsec.' % (self.RAshift, self.DECshift)) 
        else:
            self.FitsFrame.Header.add_comment('SRPComment: Astrometric solution not acceptable.') 
            self.FitsFrame.Header.add_comment('Average residual: %.2g arcsec for %d stars.' % (self.Residuals, self.NStarRes))              
        frm[0].header = self.FitsFrame.Header
        #warnings.resetwarnings()
        #warnings.filterwarnings('ignore', category=UserWarning, append=True)
        #warnings.filterwarnings('ignore', category=ResourceWarning, append=True)
        #if self.FitsFrame.BITPIX == 8:
        #    frm[0].scale('uint8')
        #elif self.FitsFrame.BITPIX == 16:
        #    frm[0].scale('uint16')
        #elif self.FitsFrame.BITPIX == 32:
        #    frm[0].scale('uint32')
        #elif self.FitsFrame.BITPIX == -32:
        #    frm[0].scale('float32')
        #elif self.FitsFrame.BITPIX == -64:
        #    frm[0].scale('float64')
        frm.writeto(nfile,overwrite=True,output_verify='ignore')
        #warnings.resetwarnings()
        #warnings.filterwarnings('always', category=UserWarning, append=True)
        #warnings.filterwarnings('always', category=ResourceWarning, append=True)






""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.2
Author  : Stefano Covino
Date    : 03/07/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : GetApPhot

Remarks :

History : (18/12/2016) First version.
        : (21/12/2016) Minor bug.
        : (03/07/2017) size must be integer in computations.
"""

import numpy as np
from astropy.io.fits import getdata



def GetFWHM(x,y,fname,size=20):
    fwhm = []
    data = getdata(fname)
    szint = int(size)
    for ii,jj in zip(x,y):
        a = np.rint(ii)-1
        b = np.rint(jj)-1
        i = a.astype(np.int)
        l = b.astype(np.int)
        image = data[l-szint:l+szint,i-szint:i+szint]
        dat=image.flatten()
        try:
            maxi = image.max()
            floor = np.median(image)
            height = maxi - floor
            #
            fwhm.append(np.sqrt(np.sum(image>floor+height/2.).flatten())[0])
        except ValueError:
            fwhm.append(-99.)
    return fwhm



#!/usr/bin/env python
"""Bootstrap setuptools installation

To use setuptools in your package's setup.py, include this
file in the same directory and add this to the top of your setup.py::

    from ez_setup import use_setuptools
    use_setuptools()

To require a specific version of setuptools, set a download
mirror, or use an alternate download directory, simply supply
the appropriate options to ``use_setuptools()``.

This file can also be run as a script to install or upgrade setuptools.
"""
import os
import shutil
import sys
import tempfile
import tarfile
import optparse
import subprocess
import platform
import textwrap

from distutils import log

try:
    from site import USER_SITE
except ImportError:
    USER_SITE = None

DEFAULT_VERSION = "2.1"
DEFAULT_URL = "https://pypi.python.org/packages/source/s/setuptools/"

def _python_cmd(*args):
    args = (sys.executable,) + args
    return subprocess.call(args) == 0

def _install(tarball, install_args=()):
    # extracting the tarball
    tmpdir = tempfile.mkdtemp()
    log.warn('Extracting in %s', tmpdir)
    old_wd = os.getcwd()
    try:
        os.chdir(tmpdir)
        tar = tarfile.open(tarball)
        _extractall(tar)
        tar.close()

        # going in the directory
        subdir = os.path.join(tmpdir, os.listdir(tmpdir)[0])
        os.chdir(subdir)
        log.warn('Now working in %s', subdir)

        # installing
        log.warn('Installing Setuptools')
        if not _python_cmd('setup.py', 'install', *install_args):
            log.warn('Something went wrong during the installation.')
            log.warn('See the error message above.')
            # exitcode will be 2
            return 2
    finally:
        os.chdir(old_wd)
        shutil.rmtree(tmpdir)


def _build_egg(egg, tarball, to_dir):
    # extracting the tarball
    tmpdir = tempfile.mkdtemp()
    log.warn('Extracting in %s', tmpdir)
    old_wd = os.getcwd()
    try:
        os.chdir(tmpdir)
        tar = tarfile.open(tarball)
        _extractall(tar)
        tar.close()

        # going in the directory
        subdir = os.path.join(tmpdir, os.listdir(tmpdir)[0])
        os.chdir(subdir)
        log.warn('Now working in %s', subdir)

        # building an egg
        log.warn('Building a Setuptools egg in %s', to_dir)
        _python_cmd('setup.py', '-q', 'bdist_egg', '--dist-dir', to_dir)

    finally:
        os.chdir(old_wd)
        shutil.rmtree(tmpdir)
    # returning the result
    log.warn(egg)
    if not os.path.exists(egg):
        raise IOError('Could not build the egg.')


def _do_download(version, download_base, to_dir, download_delay):
    egg = os.path.join(to_dir, 'setuptools-%s-py%d.%d.egg'
                       % (version, sys.version_info[0], sys.version_info[1]))
    if not os.path.exists(egg):
        tarball = download_setuptools(version, download_base,
                                      to_dir, download_delay)
        _build_egg(egg, tarball, to_dir)
    sys.path.insert(0, egg)

    # Remove previously-imported pkg_resources if present (see
    # https://bitbucket.org/pypa/setuptools/pull-request/7/ for details).
    if 'pkg_resources' in sys.modules:
        del sys.modules['pkg_resources']

    import setuptools
    setuptools.bootstrap_install_from = egg


def use_setuptools(version=DEFAULT_VERSION, download_base=DEFAULT_URL,
                   to_dir=os.curdir, download_delay=15):
    to_dir = os.path.abspath(to_dir)
    rep_modules = 'pkg_resources', 'setuptools'
    imported = set(sys.modules).intersection(rep_modules)
    try:
        import pkg_resources
    except ImportError:
        return _do_download(version, download_base, to_dir, download_delay)
    try:
        pkg_resources.require("setuptools>=" + version)
        return
    except pkg_resources.DistributionNotFound:
        return _do_download(version, download_base, to_dir, download_delay)
    except pkg_resources.VersionConflict as VC_err:
        if imported:
            msg = textwrap.dedent("""
                The required version of setuptools (>={version}) is not available,
                and can't be installed while this script is running. Please
                install a more recent version first, using
                'easy_install -U setuptools'.

                (Currently using {VC_err.args[0]!r})
                """).format(VC_err=VC_err, version=version)
            sys.stderr.write(msg)
            sys.exit(2)

        # otherwise, reload ok
        del pkg_resources, sys.modules['pkg_resources']
        return _do_download(version, download_base, to_dir, download_delay)

def _clean_check(cmd, target):
    """
    Run the command to download target. If the command fails, clean up before
    re-raising the error.
    """
    try:
        subprocess.check_call(cmd)
    except subprocess.CalledProcessError:
        if os.access(target, os.F_OK):
            os.unlink(target)
        raise

def download_file_powershell(url, target):
    """
    Download the file at url to target using Powershell (which will validate
    trust). Raise an exception if the command cannot complete.
    """
    target = os.path.abspath(target)
    cmd = [
        'powershell',
        '-Command',
        "(new-object System.Net.WebClient).DownloadFile(%(url)r, %(target)r)" % vars(),
    ]
    _clean_check(cmd, target)

def has_powershell():
    if platform.system() != 'Windows':
        return False
    cmd = ['powershell', '-Command', 'echo test']
    devnull = open(os.path.devnull, 'wb')
    try:
        try:
            subprocess.check_call(cmd, stdout=devnull, stderr=devnull)
        except:
            return False
    finally:
        devnull.close()
    return True

download_file_powershell.viable = has_powershell

def download_file_curl(url, target):
    cmd = ['curl', url, '--silent', '--output', target]
    _clean_check(cmd, target)

def has_curl():
    cmd = ['curl', '--version']
    devnull = open(os.path.devnull, 'wb')
    try:
        try:
            subprocess.check_call(cmd, stdout=devnull, stderr=devnull)
        except:
            return False
    finally:
        devnull.close()
    return True

download_file_curl.viable = has_curl

def download_file_wget(url, target):
    cmd = ['wget', url, '--quiet', '--output-document', target]
    _clean_check(cmd, target)

def has_wget():
    cmd = ['wget', '--version']
    devnull = open(os.path.devnull, 'wb')
    try:
        try:
            subprocess.check_call(cmd, stdout=devnull, stderr=devnull)
        except:
            return False
    finally:
        devnull.close()
    return True

download_file_wget.viable = has_wget

def download_file_insecure(url, target):
    """
    Use Python to download the file, even though it cannot authenticate the
    connection.
    """
    try:
        from urllib.request import urlopen
    except ImportError:
        from urllib.request import urlopen
    src = dst = None
    try:
        src = urlopen(url)
        # Read/write all in one block, so we don't create a corrupt file
        # if the download is interrupted.
        data = src.read()
        dst = open(target, "wb")
        dst.write(data)
    finally:
        if src:
            src.close()
        if dst:
            dst.close()

download_file_insecure.viable = lambda: True

def get_best_downloader():
    downloaders = [
        download_file_powershell,
        download_file_curl,
        download_file_wget,
        download_file_insecure,
    ]

    for dl in downloaders:
        if dl.viable():
            return dl

def download_setuptools(version=DEFAULT_VERSION, download_base=DEFAULT_URL,
                        to_dir=os.curdir, delay=15,
                        downloader_factory=get_best_downloader):
    """Download setuptools from a specified location and return its filename

    `version` should be a valid setuptools version number that is available
    as an egg for download under the `download_base` URL (which should end
    with a '/'). `to_dir` is the directory where the egg will be downloaded.
    `delay` is the number of seconds to pause before an actual download
    attempt.

    ``downloader_factory`` should be a function taking no arguments and
    returning a function for downloading a URL to a target.
    """
    # making sure we use the absolute path
    to_dir = os.path.abspath(to_dir)
    tgz_name = "setuptools-%s.tar.gz" % version
    url = download_base + tgz_name
    saveto = os.path.join(to_dir, tgz_name)
    if not os.path.exists(saveto):  # Avoid repeated downloads
        log.warn("Downloading %s", url)
        downloader = downloader_factory()
        downloader(url, saveto)
    return os.path.realpath(saveto)


def _extractall(self, path=".", members=None):
    """Extract all members from the archive to the current working
       directory and set owner, modification time and permissions on
       directories afterwards. `path' specifies a different directory
       to extract to. `members' is optional and must be a subset of the
       list returned by getmembers().
    """
    import copy
    import operator
    from tarfile import ExtractError
    directories = []

    if members is None:
        members = self

    for tarinfo in members:
        if tarinfo.isdir():
            # Extract directories with a safe mode.
            directories.append(tarinfo)
            tarinfo = copy.copy(tarinfo)
            tarinfo.mode = 448  # decimal for oct 0700
        self.extract(tarinfo, path)

    # Reverse sort directories.
    directories.sort(key=operator.attrgetter('name'), reverse=True)

    # Set correct owner, mtime and filemode on directories.
    for tarinfo in directories:
        dirpath = os.path.join(path, tarinfo.name)
        try:
            self.chown(tarinfo, dirpath)
            self.utime(tarinfo, dirpath)
            self.chmod(tarinfo, dirpath)
        except ExtractError as e:
            if self.errorlevel > 1:
                raise
            else:
                self._dbg(1, "tarfile: %s" % e)


def _build_install_args(options):
    """
    Build the arguments to 'python setup.py install' on the setuptools package
    """
    return ['--user'] if options.user_install else []

def _parse_args():
    """
    Parse the command line for options
    """
    parser = optparse.OptionParser()
    parser.add_option(
        '--user', dest='user_install', action='store_true', default=False,
        help='install in user site package (requires Python 2.6 or later)')
    parser.add_option(
        '--download-base', dest='download_base', metavar="URL",
        default=DEFAULT_URL,
        help='alternative URL from where to download the setuptools package')
    parser.add_option(
        '--insecure', dest='downloader_factory', action='store_const',
        const=lambda: download_file_insecure, default=get_best_downloader,
        help='Use internal, non-validating downloader'
    )
    options, args = parser.parse_args()
    # positional arguments are ignored
    return options

def main(version=DEFAULT_VERSION):
    """Install or upgrade setuptools and EasyInstall"""
    options = _parse_args()
    tarball = download_setuptools(download_base=options.download_base,
        downloader_factory=options.downloader_factory)
    return _install(tarball, _build_install_args(options))

if __name__ == '__main__':
    sys.exit(main())



""" Utility functions and classes for SRP

Context : SRP
Module  : Photometry
Version : 1.0.0
Author  : Stefano Covino
Date    : 15/02/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 
    
History : (15/02/2012) First version.

"""

import math
import numpy


def Counts2Mag (cnt, ecnt, zp=0.0):
    mag = -2.5*numpy.log10(numpy.array(cnt)) + zp
    emag = (2.5/math.log(10))*(numpy.array(ecnt)/numpy.array(cnt))
    return mag, emag


""" Utility functions and classes for SRP

Context : SRP
Module  : Frames
Version : 1.0.4
Author  : Stefano Covino
Date    : 30/05/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (28/09/2010) First version.
        : (01/10/2010) Sextractor command names.
        : (05/02/2012) Better path management.
        : (18/08/2013) ROS2 parameters.
        : (30/05/2017) Path for FITS package.
"""

import os.path

import SRP
from SRPFITS.SRPFITSPath import SRPFITSPath


# SExtractor parameter sets for photometry

BasePath        = os.path.join(SRPFITSPath(),'Data','SExtractor')

SexFName        = ("SRP.param", "SRP.conv", "SRP.nnw", "SRP.sex")

GenParSet       = ("SRPParamIn", "SRPConvIn", "SRPNnwIn", "SRPSexIn")

SRPSEXPARDICT   = {"LS-LASC":('SRPParamIn','SRPConvLSLASCIn','SRPNnwIn','SRPSexLSLASCIn'),
                    "REM-ROSS":('SRPParamIn','SRPConvIn','SRPNnwIn','SRPSexREMROSSIn'),
                    "REM-ROS2":('SRPParamIn','SRPConvIn','SRPNnwIn','SRPSexREMROS2In'),
                    "REM-REMIR":('SRPParamIn','SRPConvIn','SRPNnwIn','SRPSexREMREMIRIn'),
                    "TNG-LRS":('SRPParamIn','SRPConvIn','SRPNnwIn','SRPSexTNGLRSIn')}


# SExtractor commands
SRPsex	       = "sex"
SRPsex_cyg     = "sex.exe"

#



""" Utility functions and classes for SRP

Context : SRP
Module  : System.py
Version : 1.0.1
Author  : Stefano Covino
Date    : 29/08/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (30/05/2017) First version.
        : (29/08/2017) Bug correction.
"""


import SRPFITS

def SRPFITSPath():
    for i in SRPFITS.__path__:
        if i.find('Astro.') > 0:
            return i
    return None



""" Utility functions and classes for SRP

Context : SRP
Module  : Photometry
Version : 1.0.1
Author  : Stefano Covino
Date    : 19/04/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 
    
History : (10/11/2015) First version.
        : (19/04/2017) SRPFITS added.

"""

import numpy
from resid import resid



def centerGauss (table, x, y, r):
    if numpy.floor(x-r) < 1:
        xmin = 1
    else:
        xmin = int(numpy.floor(x-r))
    if numpy.ceil(x+r) > table.shape[1]:
        xmax = table.shape[1]
    else:
        xmax = int(numpy.ceil(x+r))
    if numpy.floor(y-r) < 1:
        ymin = 1
    else:
        ymin = int(numpy.floor(y-r))
    if numpy.ceil(y+r) > table.shape[0]:
        ymax = table.shape[0]
    else:
        ymax = int(numpy.ceil(y+r))
    field = table[(ymin-1):ymax,(xmin-1):xmax]
    init = [x-xmin,y-ymin,field[int(y-ymin),int(x-xmin)],r,r,field[0,0]]
    pars = fmin (resid, init, args=(field,), maxfun=10000, maxiter=10000, disp=0)
    return pars[0]+xmin-1, pars[1]+ymin-1, pars[2], 2.35*pars[3], 2.35*pars[4], pars[5]



""" Init file for SRPMath

Context : SRP.FITS
Module  : Math
Version : 1.0.0
Author  : Stefano Covino
Date    : 14/03/2017
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (14/03/2017) First version.
"""


__all__ = ['TriangleMatch',]





""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy
Version : 1.0.0
Author  : Stefano Covino
Date    : 02/04/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (02/04/2013) First version.

"""

import numpy



def GetSpectrumPosition (data):
    if not 1 <= data.ndim <= 2:
        return None
    elif data.ndim == 2:
        datasum = data.sum(axis=1)
    else:
        datasum = data
    num = datasum*list(range(len(datasum)))
    return num.sum()/datasum.sum()
        


""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy
Version : 1.0.1
Author  : Stefano Covino
Date    : 18/05/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (22/03/2013) First version.
        : (18/05/2017) Minor update.
"""

import numpy
from SRPFITS.Fits.GetData import GetData
from SRPFITS.Fits.GetHeaderValue import GetHeaderValue



def GetSpectrum (filename, extension=0):
    data = GetData(filename, extension)[0]
    npix = GetHeaderValue(filename,'NAXIS1',extension)[0]
    refpix = GetHeaderValue(filename,'CRPIX1',extension)[0]
    reflmb = GetHeaderValue(filename,'CRVAL1',extension)[0]
    refdl = GetHeaderValue(filename,'CDELT1',extension)[0]
    if data != None and refpix != None and reflmb != None and refdl != None:
        pxl = numpy.linspace(1,npix,npix)
        lmbd = (pxl-refpix)*refdl+reflmb
        return lmbd, data
    else:
        return None, None



""" Utility functions and classes for SRP

Context : SRP
Module  : Fits.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 01/04/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : input pars are astropy.io.fits.hdu.table.BinTableHDU
        :   t = astropy.io.fits.open('test.fits')
        :   par is t[1]

History : (01/04/2015) First version.

"""

from astropy.io import fits
#import FitsConstants

def FitsTabsAppend(orgtbl,apptbl):
    nrows1 = orgtbl.data.shape[0]
    nrows2 = apptbl.data.shape[0]
    nrows = nrows1 + nrows2
    hdu = fits.BinTableHDU.from_columns(orgtbl.columns, nrows=nrows)
    for colname in orgtbl.columns.names:
        try:
            hdu.data[colname][nrows1:] = apptbl.data[colname]
        except KeyError:
            return None
    return hdu


""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 2.0.0
Author  : Stefano Covino
Date    : 22/05/2019
E-mail  : stefano.covino@inaf.it
URL:    : http://www.brera.inaf.it/utenti/covino

Usage   :

Remarks :

History : (23/08/2017) First version.
        : (22/05/2019) Better computation of the background and its error
"""


#import SRPGW as GW
import numpy
from photutils import CircularAperture, CircularAnnulus, aperture_photometry
from astropy.stats import sigma_clipped_stats
from astropy.table import hstack
from SRPFITS.Photometry.ApErr import ApErr



#def ApyPhot(x,y,data,rds=(5,10,15),backgr=True,gain=1.,ron=0.):
#    aperture = CircularAperture((x,y), r=rds[0])
#    if backgr:
#        baperture = CircularAnnulus((x,y), r_in=rds[1], r_out=rds[2])
#        baperture_masks = baperture.to_mask(method='center')
#        baperture_data = baperture_masks[0].multiply(data)
#        mask = baperture_masks[0].data
#        baperture_data_1d = baperture_data[mask > 0]
#    else:
#        bg = 0.0
#        ebg = 0.0
#        bpix = 0
#    #
#    flux_table = aperture_photometry(data, aperture, method='subpixel')
#    npix = aperture.area()
#    #
#    if backgr:
#         _, bg,ebg = sigma_clipped_stats(baperture_data_1d)
#        bpix = baperture.area()
#    bkg_sum = bg * npix
#    flux = flux_table['aperture_sum'] - bkg_sum
#    #
#    fluxerr = ApErr(flux_table['aperture_sum']/npix,npix,bpix,bg,ebg**2,k=np.pi/2))
#    #
#    return flux,fluxerr
    


def ApyPhot(x,y,data,rds=(5,10,15),backgr=True,gain=1.,ron=0.):
    pos = [(i,l) for i,l in zip(x,y)]
    apertures = CircularAperture(pos, r=rds[0])
    if backgr:
        bapertures = CircularAnnulus(pos, r_in=rds[1], r_out=rds[2])
        bapertures_masks = bapertures.to_mask(method='center')
        bpix = bapertures.area()
    else:
        bg = 0.0
        ebg = 0.0
        bpix = 0
    #
    if backgr:
        bg = []
        ebg = []
        for mask in bapertures_masks:
            baperture_data = mask.multiply(data)
            baperture_data_1d = baperture_data[mask.data > 0]
            _, bgmed,stdev = sigma_clipped_stats(baperture_data_1d)
            bg.append(bgmed)
            ebg.append(stdev)
    bkg = numpy.array(bg)
    ebkg = numpy.array(ebg)
    #
    phot = aperture_photometry(data, apertures, method='subpixel')
    npix = apertures.area()
    phot['annulus_median'] = bkg
    phot['aper_bkg'] = bkg * npix
    phot['aper_sum_bkgsub'] = phot['aperture_sum'] - phot['aper_bkg']
    #
    flux = phot['aper_sum_bkgsub']
    if backgr:
        kp = numpy.pi/2
    else:
        kp = 0.
    fluxerr = numpy.sqrt(ApErr(phot['aperture_sum']/npix,npix,bpix,bkg,ebkg**2,k=kp))
    #
    return flux,fluxerr



""" Utility functions and classes for SRP

Context : SRP
Module  : Photometry
Version : 1.0.0
Author  : Stefano Covino
Date    : 13/11/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 
    
History : (13/11/2015) First version.

"""

import numpy
from astropy.stats import sigma_clip




def getBackground (table, x, y, rmin, rmax, sig=5.):
    if numpy.floor(x-rmax) < 1:
        xmin = 1
    else:
        xmin = int(numpy.floor(x-rmax))
    if numpy.ceil(x+rmax) > table.shape[1]:
        xmax = table.shape[1]
    else:
        xmax = int(numpy.ceil(x+rmax))
    if numpy.floor(y-rmax) < 1:
        ymin = 1
    else:
        ymin = int(numpy.floor(y-rmax))
    if numpy.ceil(y+rmax) > table.shape[0]:
        ymax = table.shape[0]
    else:
        ymax = int(numpy.ceil(y+rmax))
    field = table[ymin-1:ymax,xmin-1:xmax]
    #
    xf = numpy.arange(field.shape[1])
    xfn = numpy.arange(field.shape[0])
    yf = xfn[:,numpy.newaxis]
    #
    d = numpy.sqrt((x-(xf+xmin-0.5))**2 + (y-(yf+ymin-0.5))**2)
    l = field[(d >= rmin) & (d <= rmax)]
    ls = sigma_clip(l,sigma=sig)
    #
    mean = numpy.mean(ls)
    std = numpy.std(ls)
    return mean,std,std/numpy.sqrt(len(ls)),1.,len(ls)


""" Init file for SRPPolarimetry

Context : SRP
Module  : Polarimetry
Version : 1.6.0
Author  : Stefano Covino
Date    : 24/08/2017
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino

Usage   : to be imported

History : (15/02/2012) First named version.
        : (10/11/2015) resid, centerGauss and getBackground added.
        : (13/11/2015) MinMax added.
        : (18/11/2015) centerMoment added.
        : (19/04/2017) ApErr added.
        : (22/08/2017) DaoPhot added.
        : (23/08/2017) ApyPhot added.
        : (24/08/2017) centerObj added.
"""


__all__ = ['ApErr', 'ApyPhot', 'centerGauss', 'centerMoment', 'centerObj', 'Counts2Mag',
            'DaoPhot', 'Mag2Counts', 'getBackground', 'MinMax', 'resid']






""" Utility functions and classes for SRP

Context : SRP
Module  : Frames
Version : 1.2.0
Author  : Stefano Covino
Date    : 26/05/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (27/10/2014) First version.
        : (31/07/2015) python3 porting.
        : (16/05/2017) SRPFits porting.
        : (26/05/2017) Room for FWHM added.
"""

import os

from SRPFITS.Frames import SexConstants
from SRP.SRPSystem.Pipe import Pipe
from SRP.SRPSystem.Which import Which

from astropy.io.fits import getdata
import numpy
import sep


# Table data
class SexObjects:
    class Object:
        def __init__ (self, dlista):
            self.Id = dlista[0]
            self.X = float(dlista[1])
            self.Y = float(dlista[2])
            self.npix = float(dlista[3])
            self.ellip = 1-(float(dlista[4])/float(dlista[5]))
            self.flux = float(dlista[6])
            self.peak = float(dlista[7])
            self.flag = float(dlista[8])
            self.FWHM = -99.
            self.RA = self.X
            self.DEC = self.Y

        def __str__ (self):
            ostr = "%5s\t%7.2f\t%7.2f\t%5d\t" % (self.Id, self.X, self.Y, self.npix)
            ostr = ostr + "%5.2f\t%8.2f\t" % (self.ellip, self.flux)
            ostr = ostr + "%8.2f\t%5d\t" % (self.peak, self.flag)
            ostr = ostr + "%5.2f\t" % self.FWHM
            ostr = ostr + "%15.5f\t%15.5f" % (self.RA, self.DEC)
            return ostr+os.linesep

        def __lt__ (self, other):
            return self.flux > other.flux


    def __init__ (self, fitsfile, level=3.0):
        self.FitsFile = fitsfile
        self.Level = level
        self.ListEntries = []
        

    def FindSexObjects (self):
        # star list
        ListEntries = []
        #
        try:
            dt = getdata(self.FitsFile)
        except IOError:
            return None,SexConstants.SexFrameNotFound
        # correc binary format
        data = dt.astype(numpy.float32)
        # background evaluation and subtraction
        bkg = sep.Background(data)
        bkg.subfrom(data)
        # source extraction
        objs = sep.extract(data, self.Level * bkg.globalrms)
        # parse output
        for l in range(len(objs)):
            ListEntries.append(self.Object((l+1,objs['x'][l],objs['y'][l],objs['npix'][l],objs['b'][l],objs['a'][l],objs['flux'][l],objs['peak'][l],objs['flag'][l])))
        #
        self.ListEntries = ListEntries
        #
        return len(self.ListEntries)



""" Utility functions and classes for SRP

Context : SRP
Module  : Frames
Version : 1.1.0
Author  : Stefano Covino
Date    : 26/05/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (16/05/2017) First version.
        : (26/05/2017) Room for FWHM added.
"""

import os

from astropy.io.fits import getdata
from astropy.stats import sigma_clipped_stats
import numpy as np
from photutils import DAOStarFinder


# Table data
class DAOObjects:
    class Object:
        def __init__ (self, dlista):
            self.Id = dlista[0]
            self.X = float(dlista[1])
            self.Y = float(dlista[2])
            self.Sharpness = float(dlista[3])
            self.Roundness1 = float(dlista[4])
            self.Roundness2 = float(dlista[5])
            self.Npix = float(dlista[6])
            self.Sky = float(dlista[7])
            self.Peak = float(dlista[8])
            self.Flux = float(dlista[9])
            self.FWHM = -99
            self.RA = self.X
            self.DEC = self.Y

        def __str__ (self):
            ostr = "%5s\t%7.2f\t%7.2f\t%5.2f\t" % (self.Id, self.X, self.Y, self.Sharpness)
            ostr = ostr + "%5.2f\t%5.2f\t" % (self.Roundness1, self.Roundness2)
            ostr = ostr + "%5d\t%7.2f\t" % (self.Npix, self.Sky)
            ostr = ostr + "%7.2f\t%7.2g\t" % (self.Peak, self.Flux)
            ostr = ostr + "%5.2f\t" % (self.FWHM)
            ostr = ostr + "%15.5f\t%15.5f" % (self.RA, self.DEC)
            return ostr+os.linesep

        def __lt__ (self, other):
            return self.Flux > other.Flux


    def __init__ (self, fitsfile, level=3.0, fwhm=5):
        self.FitsFile = fitsfile
        self.Level = level
        self.FWHM = fwhm
        self.ListEntries = []
        

    def FindDAOObjects (self):
        # star list
        ListEntries = []
        #
        try:
            dt = getdata(self.FitsFile)
        except IOError:
            return None,-1
        #
        mean, median, std = sigma_clipped_stats(dt, sigma=3, iters=5)
        #
        daofind = DAOStarFinder(fwhm=self.FWHM, threshold=self.Level*std)
        objs = daofind(dt - median)
        #
        # parse output
        for l in range(len(objs)):
            ListEntries.append(self.Object((l+1,objs['xcentroid'][l],objs['ycentroid'][l],
                objs['sharpness'][l],objs['roundness1'][l],objs['roundness2'][l],
                objs['npix'][l],objs['sky'][l],objs['peak'][l],objs['flux'][l])))
        #
        self.ListEntries = ListEntries
        #
        return len(self.ListEntries)




""" Utility functions and classes for SRP

Context : SRP
Module  : Photometry
Version : 1.0.2
Author  : Stefano Covino
Date    : 12/05/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 
    
History : (18/11/2015) First version.
        : (19/04/2017) SRPFITS added.
        : (12/05/2017) Import corrected.
"""

import numpy
from SRPFITS.Photometry.getBackground import getBackground
from SRPFITS.Photometry.MinMax import MinMax



def centerMoment (table, x, y, r):
    bck = getBackground(table,x,y,0,r)
    minmax = MinMax(table,x,y,r)
    if numpy.floor(x-r) < 1:
        xmin = 1
    else:
        xmin = int(numpy.floor(x-r))
    if numpy.ceil(x+r) > table.shape[1]:
        xmax = table.shape[1]
    else:
        xmax = int(numpy.ceil(x+r))
    if numpy.floor(y-r) < 1:
        ymin = 1
    else:
        ymin = int(numpy.floor(y-r))
    if numpy.ceil(y+r) > table.shape[0]:
        ymax = table.shape[0]
    else:
        ymax = int(numpy.ceil(y+r))
    #
    field = table[(ymin-1):ymax,(xmin-1):xmax]
    bckmin1 = bck[0]+2*bck[1]
    bckmin2 = minmax[1]
    if bckmin1 > minmax[1]:
        bckmin1 = minmax[1]
    #
    nrows, ncols = field.shape
    yf, xf = numpy.mgrid[:nrows, :ncols]
    #
    l = [(field >= bckmin1) & (field <= bckmin2)]
    #
    xc = ((field[l]-bck[0])**2*xf[l]).sum()
    yc = ((field[l]-bck[0])**2*yf[l]).sum()
    tot = ((field[l]-bck[0])**2).sum()
    #
    if tot != 0.0:
            return xc/tot+xmin, yc/tot+ymin
    else:
        return x, y




""" Utility functions and classes for SRP

Context : SRP
Module  : Fits.py
Version : 1.9.0
Author  : Stefano Covino
Date    : 26/05/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (23/05/2010) First version.
        : (15/07/2010) WCS added.
        : (24/08/2010) Minor correction and source list sorting.
        : (25/08/2010) Frame size.
        : (30/08/2010) Fits Headers.
        : (04/09/2010) New importing rules.
        : (27/09/2010) Eclipse sources added.
        : (28/09/2010) Named changed from ObjList to FitsImage and adapted to native source class. 
        : (29/09/2010) Naming rationalized.
        : (04/10/2010) Possibility to remove stars close to frame border.
        : (25/10/2010) Basic statistics for frames.
        : (19/08/2013) Possiblity to read different extensions and original BITPIX saved.
        : (16/10/2013) Frame center in pixel.
        : (21/01/2014) Possibility to get statistixsw for just a subregion.
        : (27/10/2014) Sextractor source finding.
        : (16/05/2017) Minor update.
        : (26/05/2017) DAOPHOT source finding.
"""

import os

import numpy

from .GetData import GetData
from .GetHeader import GetHeader
from .GetWCS import GetWCS
from .GetHeaderValue import GetHeaderValue
from . import FitsConstants as FitsConstants
from SRPFITS.GetFWHM import GetFWHM

from SRPFITS.Frames.SourceObjectsClass import SourceObjects
from SRPFITS.Frames.SexObjectClass import SexObjects
from SRPFITS.Frames.DAOObjectClass import DAOObjects
from SRPFITS.Frames.Pixel2WCS import Pixel2WCS


class FitsImage:
    def __init__ (self, fitsfile, extension=0):
        self.Name = fitsfile
        self.Header = GetHeader(fitsfile,extension)[0]
        self.Data = GetData(fitsfile,extension)[0]
        self.WCS = GetWCS(fitsfile,extension)[0]
        self.BITPIX = GetHeaderValue(fitsfile,'BITPIX',extension)[0]
        self.NAXIS = GetHeaderValue(fitsfile,'NAXIS',extension)[0]
        self.List = []
        self.NativeSourcesFlag = False
        self.DAOSourcesFlag = False
        self.SexSourcesFlag = False


    def Sources(self, threshold=5.0, filtsing=3):
        slist = SourceObjects(self.Name)
        slist.FindObjects(self.Data, threshold, filtsing)
        srclist = []
        for i in slist.ListEntries:
            srclist.append((i.X,i.Y))
        coolist = Pixel2WCS(self.Header,srclist)
        for i,l in zip(slist.ListEntries,coolist):
            i.RA = l[0]
            i.DEC = l[1]
        self.List = slist.ListEntries
        self.NativeSourcesFlag = True
        self.DAOSourcesFlag = False
        self.SexSourcesFlag = False
        return len(self.List)
        
        
    def SortSourceList(self):
        if self.NativeSourcesFlag:
            self.List.sort()
            self.List.reverse()
        elif self.DAOSourcesFlag:
            self.List.sort()
            self.List.reverse()
        elif self.SexSourcesFlag:
            self.List.sort()
            self.List.reverse()


    def GetFWHM (self):
        X = []
        Y = []
        for i in self.List:
            X.append(i.X)
            Y.append(i.Y)
        fwhml = GetFWHM(X,Y,self.Name)
        for i,l in zip(self.List,fwhml):
            i.FWHM = l
        return len(self.List)

        
    def GetFrameSizePix(self):
        base = FitsConstants.NAxis
        sizelist = []
        try:
            naxis = self.Header[base]
        except KeyError:
            return None
        for i in range(naxis):
            baseax = '%s%s' % (base,i+1)
            try:
                value = self.Header[baseax]
            except KeyError:
                value = None
            sizelist.append(value)
        return sizelist



    def SexSources(self,level=3.0):
        elist = SexObjects(self.Name,level)
        elist.FindSexObjects()
        srclist = []
        for i in elist.ListEntries:
            srclist.append((i.X,i.Y))
        try:
            coolist = Pixel2WCS(self.Header,srclist)
        except AttributeError:
            coolist = []
            for ii in range(len(srclist)):
                coolist.append((0.,0.))
        for i,l in zip(elist.ListEntries,coolist):
            i.RA = l[0]
            i.DEC = l[1]
        self.List = elist.ListEntries
        self.SexSourcesFlag = True
        self.DAOSourcesFlag = False
        self.NativeSourcesFlag = False
        return len(self.List)



    def DAOSources(self,level=3.0):
        elist = DAOObjects(self.Name,level)
        elist.FindDAOObjects()
        srclist = []
        for i in elist.ListEntries:
            srclist.append((i.X,i.Y))
        try:
            coolist = Pixel2WCS(self.Header,srclist)
        except AttributeError:
            coolist = []
            for ii in range(len(srclist)):
                coolist.append((0.,0.))
        for i,l in zip(elist.ListEntries,coolist):
            i.RA = l[0]
            i.DEC = l[1]
        self.List = elist.ListEntries
        self.SexSourcesFlag = False
        self.DAOSourcesFlag = True
        self.NativeSourcesFlag = False
        return len(self.List)



    def CleanBorderSources(self,cleaningpercentage=1):
        framesize = self.GetFrameSizePix()
        avoidzone = (framesize[0]*cleaningpercentage/100.,framesize[1]*cleaningpercentage/100.)
        CleanList = []
        oldnobj = len(self.List)
        for entry in self.List:
            if (1+avoidzone[0] <= entry.X <= framesize[0]-avoidzone[0]) and (1+avoidzone[1] <= entry.Y <= framesize[1]-avoidzone[1]):
                CleanList.append(entry)
        self.List = CleanList
        return len(self.List),oldnobj
        
        
    def __str__(self):
        msg = ''
        # Native
        if self.NativeSourcesFlag:
            for i in range(len(self.List)):
                msg = msg + str(self.List[i])
        # daophot
        elif self.DAOSourcesFlag:
            for i in range(len(self.List)):
                msg = msg + str(self.List[i])
        # Sex
        elif self.SexSourcesFlag:
            for i in range(len(self.List)):
                msg = msg + str(self.List[i])
        return msg
    

    def Skycat(self, outname='SRP.cat'):
        msg = ''
        msg = msg + "long_name: SRP catalog for file %s\n" % (self.Name)
        msg = msg + "short_name: %s\n" % (outname)
        msg = msg + "url: ./%s\n" % (outname)
        msg = msg + "symbol: {} {circle blue} 4\n"
        msg = msg + "id_col: 0\n"
        msg = msg + "x_col: 1\n"
        msg = msg + "y_col: 2\n"
        # Native
        if self.NativeSourcesFlag:
            msg = msg + "Id\tX\tY\tRA\tDEC\tNPix\tMag\tFWHM\n"
        # daophot
        elif self.DAOSourcesFlag:
            msg = msg + "Id\tX\tY\tSharpness\tRoundness1\tRoundness2\tNpix\tSky\tPeak\tFlux\tFWHM\tRA\tDEC\n"
        # Sex
        elif self.SexSourcesFlag:
            msg = msg + "Id\tX\tY\tNpix\tEllipticity\tFlux\tPeak\tFlag\tFWHM\tRA\tDEC\n"
        
        msg = msg + "---------\n"
        msg = msg + str(self)
        msg = msg + "EOD\n"
        return msg
        
        
    def GetStats(self,region=None):
        """
        region is a list of four pixel position (lower left is 1,1)
        They are leftx, bottomy, rightx, uppery.
        """
        if region == None or len(region) != 4:
            mean = numpy.mean(self.Data)
            std = numpy.std(self.Data)
            median = numpy.median(self.Data)
            max = numpy.max(self.Data)
        else:
            try:
                mean = numpy.mean(self.Data[region[1]+1:region[3]+1,region[0]+1:region[2]+1])
                std = numpy.std(self.Data[region[1]+1:region[3]+1,region[0]+1:region[2]+1])
                median = numpy.median(self.Data[region[1]+1:region[3]+1,region[0]+1:region[2]+1])
                max = numpy.max(self.Data[region[1]+1:region[3]+1,region[0]+1:region[2]+1])
            except ValueError:
                return numpy.nan,numpy.nan,numpy.nan,numpy.nan
        #
        return mean,std,median,max



    def GetFrameCenterPix (self):
        fs = self.GetFrameSizePix()
        return int(fs[0]/2.), int(fs[1]/2.)




""" Utility functions and classes for SRP

Context : SRP
Module  : Frames.py
Version : 1.1.0
Author  : Stefano Covino
Date    : 21/11/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (22/09/2010) First version.
        : (21/11/2017) Update wioth astropy.
"""

import astLib.astWCS as astWCS
from astropy.wcs import WCS as aWCS
import math


def WCS2Pixel (fitsheader, poslist, mode='apy'):
    outlist = []
    #
    if mode == 'astlib':
        WCS = astWCS.WCS(fitsheader, mode="pyfits")
        for i in poslist:
            outlist.append(WCS.wcs2pix(i[0],i[1]))
        return outlist
    elif mode == 'apy':
        w = aWCS(fitsheader)
        for i in poslist:
            outlist.append(w.wcs_world2pix(i[0],i[1],0,ra_dec_order=True))
        return outlist



""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.1.2
Author  : Stefano Covino
Date    : 14/03/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (27/09/2010) First version.
        : (13/10/2010) Quicker triangle scan.
        : (26/08/2013) More control on accepted matches.
        : (14/03/2017) Possibility to remove not useful iterations 
            and porting to SRP.FITS.
"""

import math

from SRP.SRPMath.TriangleClass import Triangle
from SRP.SRPMath.CartesianCoordAmpRotoTranslation import CartesianCoordAmpRotoTranslation
from SRP.SRPStatistics.CoordDistanceMinimization import CoordDistanceMinimization
from SRP.SRPMath.AngleRange import AngleRange



def TriangleMatch (reflist,objlist,angtol=0.5,disttol=5.,TrueSize=None,TrueAng=None):
#    print objlist
#    print reflist
    if len(reflist) >=3 and len(objlist) >=3:
        #
        if len(reflist) >= 5:
            reqmtch = 5
        else:
            reqmtch = len(reflist)
        #
        for i in range(len(reflist)):
            for ii in range(i,len(reflist)):
                for iii in range(ii,len(reflist)):
                    for l in range(len(objlist)):
                        for ll in range(l,len(objlist)):
                            for lll in range(ll,len(objlist)):
                                if i != ii and ii != iii and i != iii and l != ll and ll != lll and l != lll:
                                    #print i,ii,iii,l,ll,lll
                                    triref = Triangle(reflist[i],reflist[ii],reflist[iii],angtol)
                                    triobj = Triangle(objlist[l],objlist[ll],objlist[lll],angtol)
                                    if not (triobj.IsSymmTriangle() or triref.IsSymmTriangle()):
                                        # cannot work with symmetric triangles
                                        # Select not too small or elongated triangles
                                        if min(triobj.Sizes) >= 10.0 and min(triobj.Angles) > 10.0 and triobj.Commensurabili(triref):
                                            if triobj.Rotable(triref):
                                                # with a good candidate compute pairing data
                                                sf = triobj.SizeFactor(triref)
                                                rf = triobj.RotationFactor(triref)
                                                sh = triobj.ShiftFactor(triref)
                                                if TrueSize != None:
                                                    if TrueSize/sf > 1.1 or TrueSize/sf < 0.9:
                                                        continue
                                                if TrueAng != None:
                                                    if abs(AngleRange(TrueAng)-AngleRange(rf)) > 10.:
                                                        continue
                                                #print i,ii,iii,l,ll,lll
                                                #print ("sf,rf,sz", sf,rf,sh)
                                                #print "objs",triobj.Vertici[0],triobj.Vertici[1],triobj.Vertici[2]
                                                #print "refs",triref.Vertici[0],triref.Vertici[1],triref.Vertici[2]
                                                #print "obj-ang", triobj.Angles
                                                #print "ref-ag", triref.Angles
                                                #print "obj-siz", triobj.Sizes
                                                #print "ref-siz", triref.Sizes
                                                #print "ccns",triobj.ConnVert
                                                # find acceptable error
                                                tempinp = []
                                                tempout = []
                                                for zz in triobj.ConnVert:
                                                    tempinp.append(triobj.Vertici[zz[0]].Coord)
                                                    tempout.append(triref.Vertici[zz[1]].Coord)
                                                res = CoordDistanceMinimization(tempout,tempinp,[sh[0],sh[1],rf,1./sf])   
                                                # acceptable tolerance for match 
                                                minassdist = disttol*res[1]
                                                #print minassdist, res
                                                #print
                                                #    
                                                #minassdist = 0
                                                #for zz in triobj.ConnVert:
                                                #    miecord = CartesianCoordAmpRotoTranslation(triobj.Vertici[zz[0]].Coord,sh,rf,1./sf)
                                                #    miecorddist = math.sqrt((miecord[0]-triref.Vertici[zz[1]].Coord[0])**2 + (miecord[1]-triref.Vertici[zz[1]].Coord[1])**2)
                                                #    minassdist = minassdist + miecorddist 
                                                #    #print miecorddist,triobj.Vertici[zz[0]].Coord, miecord, triref.Vertici[zz[1]].Coord
                                                #minassdist = minassdist*5./3.
                                                # with a basic solution try to find a general one
                                                # pair points
                                                nowinp = []
                                                nowout = []
                                                assobj = [False for c in reflist]
                                                for pp in range(len(objlist)):
                                                    # convert coordinates from obj to ref
                                                    miecord = CartesianCoordAmpRotoTranslation(objlist[pp],(res[0][0],res[0][1]),res[0][2],res[0][3])
                                                    #print "miecord",miecord
                                                    mindist = 1e12
                                                    objmem = 0
                                                    for qq in range(len(reflist)):
                                                        #print pp, qq, assobj[qq]
                                                        #print "obj",objlist[pp],"ref",reflist[qq]
                                                        # if point is not assigned and close enough to a frame point go on
                                                        if not assobj[qq]:
                                                            mdist = math.sqrt((miecord[0]-reflist[qq][0])**2 + (miecord[1]-reflist[qq][1])**2)
                                                            if mdist < mindist:
                                                                mindist = mdist
                                                                objmem = qq
                                                    if mindist <= minassdist:
                                                        assobj[objmem] = True
                                                        nowinp.append(pp)
                                                        nowout.append(objmem)
                                                    #print "dist",mindist,minassdist,objmem
                                                # now we check how many associated points we have
                                                #print len(nowout),reqmtch
                                                if len(nowout) >= reqmtch:
                                                    return nowinp,nowout,res
                                            else:
                                                pass
#                                                print "Not rotable."
    return None
    



""" Utility functions and classes for SRP

Context : SRP
Module  : Fits.py
Version : 1.3.1
Author  : Stefano Covino
Date    : 18/05/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : AddHeaderEntry (fitsfile, keylist, entrylist, commentlist, outfilename=None)
            "fitfile" is the FITS file name.
            "keylist" is a list of keywords to be added to the FITS file headers.
            "entrylist" is a list of the same length than "keylist" with keywords values.
            "commentlist" is a list of the same length of "keylist" with comments (units, etc.)
                to the new headers.
            "outfilename" optional filename for output. Else input file file is overwritten.
            
            Function returns two values: (res, code). If res is False code reports the problem 
                (codes are in SRPFITS.FitsConstants). Else res is True.

Remarks :

History : (12/10/2010) First version.
        : (25/04/2011) Output and input file names can be different.
        : (27/04/2011) Always list in output. Keywords longer than 8 characters.
        : (14/02/2012) Possibilty to write in other extensions.
        : (31/03/2013) Value considered float unless it is not possible to convert it.
        : (31/07/2015) python3 porting.
        : (18/05/2017) astropy.io.fits.
"""

import warnings

from astropy.io import fits
from . import FitsConstants

def AddHeaderEntry (fitsfile, keylist, entrylist, commentlist, outfilename=None, ext=0):
    try:
        hdr = fits.open(fitsfile)
    except IOError:
        return False,FitsConstants.FitsFileNotFound
    heder = hdr[ext].header    
    for i,l,m in zip(keylist,entrylist,commentlist):
        try:
            lf = float(l)
        except ValueError:
            lf = l
        try:
            heder.set(i,lf,m)
        except ValueError:
            heder.set(i.split()[0][:8],l,m)
    #
    warnings.resetwarnings()
    warnings.filterwarnings('ignore', category=UserWarning, append=True)
    warnings.filterwarnings('ignore', category=ResourceWarning, append=True)
    if outfilename == None:
        hdr.writeto(fitsfile,overwrite=True)
    else:
        hdr.writeto(outfilename,overwrite=True)
    warnings.resetwarnings() 
    warnings.filterwarnings('always', category=UserWarning, append=True)
    warnings.filterwarnings('always', category=ResourceWarning, append=True)
    return True,FitsConstants.FitsOk



""" Utility functions and classes for SRP

Context : SRP
Module  : Frames.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 23/05/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (23/05/2010) First version.

"""

import astLib.astWCS as astWCS
import math


def Pixel2WCS (fitsheader, poslist):
    outlist = []
    WCS = astWCS.WCS(fitsheader, mode="pyfits")
    for i in poslist:
        outlist.append(WCS.pix2wcs(i[0],i[1]))
    return outlist
    


""" Utility functions and classes for SRP

Context : SRP
Module  : Fits.py
Version : 1.2.1
Author  : Stefano Covino
Date    : 27/04/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (24/05/2010) First version.
        : (15/07/2010) New constants.
        : (30/08/2010) Astrometry headers.
        : (04/09/2010) New headers.
        : (27/09/2010) Eclipse constants.
        : (29/09/2010) No more eclipse constants.
        : (17/11/2010) Comments for WCS header.
        : (20/12/2010) Not known WCS entry.
        : (27/04/2011) Constant for "no problem".

"""

# Header
FitsFileNotFound    =   -1
FitsHeaderNotFound  =   -2
FitsHeaderFound     =   1
FitsOk              =   0

# Data
FitsDataSetFound    =   2
FitsDataSetNotFound =   -3

# WCS
FitsWCSFound    = 3
FitsWCSNotKnown = -3


# Finding-chart output
FCExt = '_fc.png'

# Source output
RegData = '_src.dat'
SkyData = '_src.skycat'

# Size Headers
NAxis       =   'NAXIS'

# Astrometry Headers
CRPIX1  =   'CRPIX1'
CRPIX2  =   'CRPIX2'
CRVAL1  =   'CRVAL1'
CRVAL2  =   'CRVAL2'
CTYPE1  =   'CTYPE1'
CTYPE2  =   'CTYPE2'
RAType  =   'RA---TAN'           
DECType =   'DEC--TAN'    
CDELT1  =   'CDELT1'
CDELT2  =   'CDELT2'
PC11    =   'PC1_1'
PC12    =   'PC1_2'
PC21    =   'PC2_1'
PC22    =   'PC2_2'
CD11    =   'CD1_1'
CD12    =   'CD1_2'
CD21    =   'CD2_1'
CD22    =   'CD2_2'
RADECSys    =   'RADECSYS'
RADECSysVal =   'FK5'
AngUnit1    =   'CUNIT1'
AngUnit2    =   'CUNIT2'
AngUnit1Val =   'Degrees '
AngUnit2Val =   'Degrees '
InAxesUnit  =   'PIXEL'
RA      =   'RA'
DEC     =   'DEC'
#
CRPIX1Cmt   =   'reference X pixel'
CRPIX2Cmt   =   'reference Y pixel'
CRVALCmt    =   'sky coords at reference pixel'
CTYPECmt    =   'sky projection'
CDELT1Cmt   =   'increment for X pixel'
CDELT2Cmt   =   'increment for Y pixel'
PCCmt       =   'rotation matrix values'
AngUnCmt    =   'sky coords units'
#


""" Utility functions and classes for SRP

Context : SRP
Module  : Frames
Version : 1.0.0
Author  : Stefano Covino
Date    : 29/09/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (29/09/2010) First version.


"""


# Eclipse package
SRPpeak                 = "peak"
EclipseNotFound         =   -1
EclipseNotExecutable    =   -2


""" Utility functions and classes for SRP

Context : SRP
Module  : Photometry
Version : 1.0.0
Author  : Stefano Covino
Date    : 13/11/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 
    
History : (13/11/2015) First version.

"""

import numpy


def MinMax (table, x, y, r):
    if numpy.floor(x-r) < 1:
        xmin = 1
    else:
        xmin = int(numpy.floor(x-r))
    if numpy.ceil(x+r) > table.shape[1]:
        xmax = table.shape[1]
    else:
        xmax = int(numpy.ceil(x+r))
    if numpy.floor(y-r) < 1:
        ymin = 1
    else:
        ymin = int(numpy.floor(y-r))
    if numpy.ceil(y+r) > table.shape[0]:
        ymax = table.shape[0]
    else:
        ymax = int(numpy.ceil(y+r))
    #
    field = table[(ymin-1):ymax,(xmin-1):xmax]
    #
    maxf = field.max()
    minf = field.min()
    cMy,cMx = numpy.unravel_index(field.argmax(), field.shape)
    cmy,cmx = numpy.unravel_index(field.argmin(), field.shape)
    #
    return minf, maxf, (cmx+xmin,cmy+ymin), (cMx+xmin,cMy+ymin)




""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.0
Author  : Stefano Covino
Date    : 23/08/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   :

Remarks :

History : (23/08/2017) First version.
"""


#import SRPGW as GW
import numpy
from photutils import CircularAperture, CircularAnnulus, aperture_photometry
from astropy.table import hstack



def ApyPhot(x,y,data,rds=(5,10,15),backgr=True,gain=1.,ron=0.):
    aperture = CircularAperture((x,y), r=rds[0])
    if backgr:
        baperture = CircularAnnulus((x,y), r_in=rds[1], r_out=rds[2])
        back_table = aperture_photometry(data, baperture, method='subpixel')
    else:
        bg = 0.0
        ebg = 0.0
    #
    flux_table = aperture_photometry(data, aperture, method='subpixel')
    npix = aperture.area()
    #
    if backgr:
        phot_table = hstack([flux_table, back_table], table_names=['raw', 'bkg'])
        bpix = baperture.area()
        bg = phot_table['aperture_sum_bkg'] / bpix
    bkg_sum = bg * npix
    flux = phot_table['aperture_sum_raw'] - bkg_sum
    #
    fluxerr = numpy.sqrt(numpy.fabs(phot_table['aperture_sum_raw'])*gain + numpy.fabs(bkg_sum)*gain + (npix+bpix)*ron)
    #
    return flux,fluxerr
    



""" Utility functions and classes for SRP

Module  : Frames.py
Version : 1.0.1
Author  : Stefano Covino
Date    : 15/05/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (31/08/2012) First version.
        : (15/05/2017) Minor update.
"""

from SRPFITS.Fits.GetWCS import GetWCS


def getCenterRADEC (fitsfile, xpix=None, ypix=None):
    wcs = GetWCS(fitsfile)[0]
    if wcs != None:
        if xpix == None or ypix == None:
            return wcs.getCentreWCSCoords()
        else:
            return wcs.pix2wcs(xpix,ypix)
    else:
        return None



""" Utility functions and classes for SRP

Context : SRP
Module  : Photometry
Version : 1.0.0
Author  : Stefano Covino
Date    : 19/04/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : 
            gain: gain in "electron / pixel data units"
            NA: number of pixel in source aperture
            NB: number of pixels in background annulus
            ni: depth - of - coverage at pixel i; N i > 1 if image is a co-add
            SI: signal per pixel in image data units
            BM: estimated background per pixel in annulus (either mean or median) 
                    if mean than k = 1
                    else if median  k = np.pi/2
                    else if no background k = 0
            SB2: variance in sky background annulus in [image units]^2 / pixel

Remarks : adapted from a text by F. Masci, v. 1.0.
    
History : (19/04/2017) First version.

"""


import numpy as np


def ApErr (SI, NA, NB, BM, SB2, ni=1, gain=1, k=1):
    ft = (1./gain)*NA*(SI-BM)/ni
    st = (NA+k*NA**2/NB)*SB2
    ftw = np.where(ft < 0, 0., ft)
    return ftw+st



""" Init file for SRPFrames

Context : SRP
Module  : Frames
Version : 1.1.0
Author  : Stefano Covino
Date    : 16/05/2017
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (31/08/2012) First named version.
        : (16/05/2017) DAOObjectClass added.
"""



__all__ = ['AstrometryClass', 'DAOObjectClass', 'EclipseConstants', 'EclipseObjectClass',
           'getCenterRADEC', 'Pixel2WCS', 'SexConstants', 'SexObjectClass',
           'SExtractorConstants', 'SourceObjectsClass', 'WCS2Pixel']





""" Utility functions and classes for SRP

Context : SRP
Module  : Fits.py
Version : 1.2.2
Author  : Stefano Covino
Date    : 18/05/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : AddHeaderComment (fitsfile, commentlist, outfilename=None)
            "fitfile" is the FITS file name.
            "commentlist" is a list with the comments to add.
            "outfilename" optional filename for output. Else input file file is overwritten.
            
            Function returns two values: (res, code). If res is False code reports the problem 
                (codes are in SRPFITS.FitsConstants). Else res is True.

Remarks :

History : (01/10/2010) First version.
        : (25/04/2011) Input and output file names can be different.
        : (27/04/2011) Always list in output.
        : (21/07/2014) Better management of non-standard FITS headers.
        : (31/07/2015) python3 porting.
        : (18/05/2017) astropy.io.fits
"""

import warnings

from astropy.io import fits
from . import FitsConstants

def AddHeaderComment (fitsfile, commentlist, outfilename=None):
    try:
        hdr = fits.open(fitsfile)
    except IOError:
        return False,FitsConstants.FitsFileNotFound
    heder = hdr[0].header
    for i in commentlist:
        heder.add_comment(i)
    #
    warnings.resetwarnings()
    warnings.filterwarnings('ignore', category=UserWarning, append=True)
    warnings.filterwarnings('ignore', category=ResourceWarning, append=True)
    if outfilename == None:
        hdr.writeto(fitsfile,overwrite=True,output_verify='ignore')
    else:
        hdr.writeto(outfilename,overwrite=True,output_verify='ignore')
    warnings.resetwarnings() 
    warnings.filterwarnings('always', category=UserWarning, append=True)
    warnings.filterwarnings('always', category=ResourceWarning, append=True)
    hdr.close()
    return True,FitsConstants.FitsOk



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPFITS
Version : 1.1.0
Author  : Stefano Covino
Date    : 40/08/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   :

Remarks :

History : (22/08/2017) First version.
        : (30/08/2017) More sky algorithms.

"""


import numpy
from PythonPhot import aper


def DaoPhot(x,y,data,rds=(5,10,15),backgr=True,gain=1.,ron=0.,skyalg='mmm'):
    if backgr:
        mag,magerr,flux,fluxerr,sky,skyerr,badflag,outstr = aper.aper(data,x,y,phpadu=gain,apr=rds[0],zeropoint=30,skyrad=[rds[1],rds[2]],readnoise=ron,skyalgorithm=skyalg)
    else:
        mag,magerr,flux,fluxerr,sky,skyerr,badflag,outstr = aper.aper(data,x,y,phpadu=gain,apr=rds[0],zeropoint=30,skyrad=[rds[1],rds[2]],setskyval=0.0,readnoise=ron)
    #
    return flux.flatten(),fluxerr.flatten()
    





""" Utility functions and classes for SRP

Context : SRP
Module  : Frames.py
Version : 1.5.0
Author  : Stefano Covino
Date    : 26/05/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (25/06/2010) First version.
        : (24/08/2010) Possibility to choose the minimum number of pixel per source.
        :               and management of the 'total flux negative' case.
        : (28/09/2010) Upgraded code as a class.
        : (18/10/2010) Coding correction.
        : (04/04/2013) New native source extraction algorithm.
        : (31/07/2015) python3 porting.
        : (26/05/2017) Room for FWHM added.
"""


import scipy.ndimage.measurements as snm
import scipy.ndimage as nd
import astLib.astStats as aa
import math, os, warnings
import numpy



# Source data
class SourceObjects:
    class Object:
        def __init__ (self, dlista):
            self.Id = dlista[0]
            self.X = float(dlista[1])
            self.Y = float(dlista[2])
            self.Npix = float(dlista[3])
            self.Mag = float(dlista[4])
            self.FWHM = -99
            self.RA = self.X
            self.DEC = self.Y


        def __str__ (self):
            msg = ''
            msg = msg + '%10d\t%10.3f\t%10.3f\t' % (self.Id, self.X, self.Y)
            msg = msg + '%10.6f\t%10.6f\t%5d\t%5.1f\t' % (self.RA, self.DEC, self.Npix, self.Mag)
            msg = msg + '%5.2f' % (self.FWHM)
            msg = msg + os.linesep
            return msg

        def __lt__ (self, other):
            return self.Mag < other.Mag


    def __init__ (self, fitsfile):
        self.FitsFile = fitsfile
        self.ListEntries = []

    
    def FindObjectsMy (self, table, sigma=5, filtsing=3):
        warnings.resetwarnings()
        warnings.filterwarnings('ignore', category=RuntimeWarning, append=True)
        # statistiche iniziali
        scistats = aa.clippedMeanStdev(table)
        sci_clip = table >= sigma*scistats['clippedMean']
        sci_labels,sci_num = nd.label(sci_clip)
        counts = nd.sum(table,sci_labels,list(range(sci_num)))
        pos = nd.center_of_mass(table,sci_labels,list(range(sci_num)))
        mag = -2.5*numpy.log10(counts)
        warnings.resetwarnings()
        warnings.filterwarnings('always', category=RuntimeWarning, append=True)
        finlist = []
        for i in range(len(counts)):
            if not (numpy.isnan(pos[i][0]) or numpy.isnan(pos[i][1]) or numpy.isnan(mag[i]) or numpy.isinf(mag[i])):
                finlist.append(self.Object([i+1,pos[i][0],pos[i][1],0,mag[i]]))
        #
        self.ListEntries = finlist
        #
        return len(self.ListEntries)



    def FindObjects (self, table, sigma=5, filtsing=3):
        # statistiche iniziali
        tt = aa.clippedMeanStdev(table)
        totbck = tt['clippedMean']
        totstd = tt['clippedStdev']

        # selezione pixel
        datapos = []
        for x in range(table.shape[1]):
            for y in range(table.shape[0]):
                if table[y][x] > totbck+sigma*totstd:
                    datapos.append((y,x))


        # Scelta pixel connessi
        totloc = []
        for i in datapos:
            cn = []
            cn.append(i)
            for l in datapos:
                if (i[0]-l[0])**2 + (i[1]-l[1])**2 <= 2.0:
                    if l not in cn:
                        cn.append(l)
            ass = False
            for p in range(len(totloc)):
                for q in cn:
                    if q in totloc[p]:
                        totloc[p] = totloc[p] + cn
                        ass = True
                        break
                if ass == True:
                    break
            else:
                totloc.append(cn)


        # Eliminazione doppioni
        finloc = []
        for t in totloc:
            finpos = []
            for p in t: 
                if p not in finpos:
                    finpos.append(p)
            finloc.append(finpos)


        # Calcolo baricentro e pseudomagnitudine  
        finlist = []
        objl = 0
        for fc in finloc:
            minx = table.shape[1]
            miny = table.shape[0]
            maxx = 0
            maxy = 0
            for e in fc:
                if e[0] < miny:
                    miny = e[0]
                if e[0] > maxy:
                    maxy = e[0]
                if e[1] < minx:
                    minx = e[1]
                if e[1] > maxx:
                    maxx = e[1]
            cm = snm.center_of_mass(table[miny:maxy+1,minx:maxx+1])
            sm = snm.sum((table[miny:maxy+1,minx:maxx+1]))
            if len(fc) >= filtsing:
                logarg = sm-(maxy+1-miny)*(maxx+1-minx)*totbck
                if logarg < 0.0:
                    signum = -1.0
                elif logarg == 0.0:
                    signum = 1.0
                    logarg = 1e-30
                else:
                    signum =  1.0
                objl = objl + 1
                finlist.append(self.Object([objl,minx+cm[1]+1,miny+cm[0]+1,len(fc),(-2.5*signum*math.log10(math.fabs(logarg)))]))
        
        #
        self.ListEntries = finlist
        #
        return len(self.ListEntries)



""" Utility functions and classes for SRP

Context : SRP
Module  : Fits.py
Version : 1.1.1
Author  : Stefano Covino
Date    : 18/05/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (27/05/2010) First version.
        : (31/07/2015) python3 porting.
        : (18/05/2017) Close open file anyway.
"""

from astropy.io import fits
import tempfile
from . import FitsConstants

def IsFits(fitsfile):
    try:
        hdr = fits.open(fitsfile)
        fitst = True
    except (IOError,OSError):
        hdr = tempfile.TemporaryFile()
        fitst = False
    finally:
        hdr.close()
        return fitst




from setuptools import setup, find_packages
from os import path
import glob
import SRPFITS as FITS
import os

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


# Look for script files
lscr = glob.glob(os.path.join('Scripts', 'SRP*'))
lscrex = []
for i in lscr:
    if os.path.splitext(i)[1] == '':
        lscrex.append(i)


# Look for data files
lsdt = glob.glob(os.path.join('SRPFITS/Data', '*'))
lsdtex = []
for i in lsdt:
    if os.path.splitext(i)[1] == '':
        lsdtex.append(i)



setup(
    name='SRPAstro.FITS',
    version=FITS.__version__,
    description='Tools for handling FITS files under SRP',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://pypi.python.org/pypi/SRPAstro',
    author='Stefano Covino',
    author_email='stefano.covino@inaf.it',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Astronomy',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 3',
        ],
    keywords='astronomy data analysis',
    packages=find_packages(),
    python_requires='>=3',
    package_data={'SRPFITS':lsdtex},
    scripts=lscrex,
    install_requires=['SRPAstro >= 4.4', 'sep', 'photutils', 'astropy'],
    ) 




""" Init file for Fits

Context : SRP
Module  : SRPFits
Version : 1.0.4
Author  : Stefano Covino
Date    : 27/05/2017
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (28/09/2010) First named version.
        : (01/10/2010) New function.
        : (06/01/2010) New function: AddHeaderEntry.
        : (01/04/2015) FitsTabsAppend added.
        : (27/05/2017) New functions added.
"""



__all__ = ['AddHeaderComment', 'AddHeaderEntry', 'FitsConstant', 'FitsImageClass',
           'FitsTabsAppend', 'GetData', 'GetHeader', 'GetHeaderValue', 'GetSpectrum',
           'GetSpectrumPosition', 'GetWCS', 'IsFits']





""" Utility functions and classes for SRP

Context : SRP
Module  : Frames.py
Version : 1.1.1
Author  : Stefano Covino
Date    : 16/05/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (27/09/2010) First version.
        : (29/09/2010) More ordered importing.
        : (31/07/2015) python3 porting.
        : (16/05/2017) SRPFits porting.
"""

import os

from SRPFITS.Frames import EclipseConstants
from SRP.SRPSystem.Pipe import Pipe
from SRP.SRPSystem.Which import Which



# Peak data
class EclipseObjects:
    class Object:
        def __init__ (self, dlista):
            self.Id = dlista[0]
            self.X = float(dlista[1])
            self.Y = float(dlista[2])
            self.pix = float(dlista[3])
            self.mean = float(dlista[4])
            self.dev = float(dlista[5])
            self.med = float(dlista[6])
            self.min = float(dlista[7])
            self.max = float(dlista[8])
            self.fx = float(dlista[9])
            self.fy = float(dlista[10])
            self.FWHM = float(dlista[11])
            self.flux = float(dlista[12])
            self.RA = self.X
            self.DEC = self.Y

        def __str__ (self):
            ostr = "%5s\t%7.2f\t%7.2f\t%5d\t" % (self.Id, self.X, self.Y, self.pix)
            ostr = ostr + "%8.2f\t%8.2f\t" % (self.min, self.max)
            ostr = ostr + "%7.2f\t%7.2f\t%7.2f\t%10.3f\t" % (self.fx, self.fy, self.FWHM, self.flux)
            ostr = ostr + "%15.5f\t%15.5f" % (self.RA, self.DEC)
            return ostr+os.linesep

        def __lt__ (self, other):
            return self.flux > other.flux


    def __init__ (self, fitsfile, level=2.0):
        self.FitsFile = fitsfile
        self.Level = level
        self.ListEntries = []
        

    def FindEclipseObjects (self):
        # Eclipse required
        if Which(EclipseConstants.SRPpeak) == None:
            return None,EclipseConstants.EclipseNotFound
        # star list
        ListEntries = []
        argl = " -f '5 10 15' -F -P '5 10 15' -m clip -k %s " % self.Level
        cmd = EclipseConstants.SRPpeak+argl+self.FitsFile
        res = Pipe(cmd)
        if res == None:
            return None,EclipseConstants.EclipseNotExecutable
        # parse output
        for l in res.decode().split(os.linesep):
            if len(l) > 2:
                try:
                    ListEntries.append(self.Object(l.split()))
                except ValueError:
                    pass
        # 
        self.ListEntries = ListEntries
        #
        return len(self.ListEntries)



""" Utility functions and classes for SRP

Context : SRP
Module  : Frames
Version : 1.0.0
Author  : Stefano Covino
Date    : 27/10/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (27/10/2014) First version.


"""


# Sex constants
SexFrameNotFound         =   -1



""" Utility functions and classes for SRP

Context : SRP
Module  : Fits.py
Version : 1.3.0
Author  : Stefano Covino
Date    : 31/07/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (21/05/2010) First version.
        : (14/02/2012) Possibility to read from other extensions.
        : (31/07/2015) python3 porting.
"""

from astropy.io import fits
from . import FitsConstants

def GetHeaderValue (fitsfile, header, ext=0):
    try:
        hdr = fits.open(fitsfile)
    except IOError:
        return None,FitsConstants.FitsFileNotFound
    hdr[ext].verify('silentfix+ignore')
    try:
        headval = hdr[ext].header[header]
    except KeyError:
        return None,FitsConstants.FitsHeaderNotFound
    hdr.close()
    return headval,FitsConstants.FitsHeaderFound
    

    



""" Utility functions and classes for SRP

Context : SRP
Module  : Photometry
Version : 1.0.0
Author  : Stefano Covino
Date    : 15/02/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 
    
History : (15/02/2012) First version.

"""


import math
import numpy


def Mag2Counts (mag, emag, zp=0.0):
    cnt = 10**(-0.4*(numpy.array(mag)-zp))
    ecnt = (numpy.array(emag)*cnt)/(2.5/math.log(10.0))
    return cnt, ecnt


""" Utility functions and classes for SRP

Context : SRP
Module  : Photometry
Version : 1.0.0
Author  : Stefano Covino
Date    : 10/11/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 
    
History : (10/11/2015) First version.

"""

import numpy
from SRP.SRPMath.BiGauss import BiGauss



def resid (pars,field):
    x0 = pars[0]
    y0 = pars[1]
    A = pars[2]
    sx = numpy.abs(pars[3])
    sy = numpy.abs(pars[4])
    B = pars[5]
    if x0 < 0.0:
        x0 = 0.0
    elif x0 > field.shape[1]:
        x0 = field.shape[1]
    if y0 < 0.0:
        y0 = 0.0
    elif y0 > field.shape[0]:
        y0 = field.shape[0]
    #
    y = BiGauss(field,x0,y0,A,sx,sy,B)
    restot = ((y-field)**2).sum()
    return restot



""" Utility functions and classes for SRP

Context : SRP
Module  : Fits.py
Version : 1.2.0
Author  : Stefano Covino
Date    : 16/12/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (15/07/2010) First version.
        : (20/12/2010) Possibility of badly formatted ECS header included.
        : (23/08/2013) NUMPY_MODE in wcs.
        : (16/12/2014) Manage wrong FITS headers.
"""

from . import FitsConstants
import astLib.astWCS as aw
from .GetHeader import GetHeader
from . import FitsConstants



def GetWCS (fitsfile, extension=0, orig00=False):
    heder = GetHeader(fitsfile,extension)
    if heder[1] == FitsConstants.FitsHeaderFound:
        try:
            wcs = aw.WCS(heder[0],mode='pyfits')
            wcs.NUMPY_MODE = orig00
        except IOError:
            return None,FitsConstants.FitsFileNotFound
        except ValueError:
            return None,FitsConstants.FitsWCSNotKnown
        return wcs,FitsConstants.FitsWCSFound
    else:
        return None,FitsConstants.FitsFileNotFound




""" Utility functions and classes for SRP

Context : SRP
Module  : Photometry
Version : 1.0.1
Author  : Stefano Covino
Date    : 19/04/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 
    
History : (10/11/2015) First version.
        : (19/04/2017) SRPFITS added.

"""

import numpy as np
from SRPFITS.Photometry.centerMoment import centerMoment
from photutils.centroids import centroid_com
from photutils.centroids import centroid_1dg
from photutils.centroids import centroid_2dg



def centerObj (table, x, y, r, mode='my'):
    if mode == 'my':
        return centerMoment(table, x, y, r)
    else:
        if np.floor(x-r) < 1:
            xmin = 1
        else:
            xmin = int(np.floor(x-r))
        if np.ceil(x+r) > table.shape[1]:
            xmax = table.shape[1]
        else:
            xmax = int(np.ceil(x+r))
        if np.floor(y-r) < 1:
            ymin = 1
        else:
            ymin = int(np.floor(y-r))
        if np.ceil(y+r) > table.shape[0]:
            ymax = table.shape[0]
        else:
            ymax = int(np.ceil(y+r))
        #
        field = table[(ymin-1):ymax,(xmin-1):xmax]
        if mode == 'apymom':
            xr,yr = centroid_com(field)
            return xr+xmin, yr+ymin
        if mode == 'apy1dg':
            xr,yr = centroid_1dg(field)
            return xr+xmin, yr+ymin
        if mode == 'apy2dg':
            xr,yr = centroid_2dg(field)
            return xr+xmin, yr+ymin
        else:
            return x,y
